@@ -4,7 +4,7 @@ import textwrap
 from collections import OrderedDict
 from contextlib import contextmanager
 
-from conan.tools.microsoft.subsystems import deduce_subsystem, WINDOWS
+from conans.client.subsystems import deduce_subsystem, WINDOWS, subsystem_path
 from conans.errors import ConanException
 from conans.util.files import save
 
@@ -13,34 +13,42 @@ class _EnvVarPlaceHolder:
     pass
 
 
-def environment_wrap_command(env_filenames, cmd, subsystem=None, cwd=None):
-    from conan.tools.microsoft.subsystems import subsystem_path
-    assert env_filenames
+def environment_wrap_command(env_filenames, env_folder, cmd, subsystem=None,
+                             accepted_extensions=None):
+    if not env_filenames:
+        return cmd
     filenames = [env_filenames] if not isinstance(env_filenames, list) else env_filenames
-    bats, shs = [], []
-
-    cwd = cwd or os.getcwd()
+    bats, shs, ps1s = [], [], []
 
+    accept = accepted_extensions or ("ps1", "bat", "sh")
+    # TODO: This implemantation is dirty, improve it
     for f in filenames:
-        f = f if os.path.isabs(f) else os.path.join(cwd, f)
+        f = f if os.path.isabs(f) else os.path.join(env_folder, f)
         if f.lower().endswith(".sh"):
-            if os.path.isfile(f):
+            if os.path.isfile(f) and "sh" in accept:
                 f = subsystem_path(subsystem, f)
                 shs.append(f)
         elif f.lower().endswith(".bat"):
-            if os.path.isfile(f):
+            if os.path.isfile(f) and "bat" in accept:
                 bats.append(f)
+        elif f.lower().endswith(".ps1") and "ps1" in accept:
+            if os.path.isfile(f):
+                ps1s.append(f)
         else:  # Simple name like "conanrunenv"
             path_bat = "{}.bat".format(f)
             path_sh = "{}.sh".format(f)
-            if os.path.isfile(path_bat):
+            path_ps1 = "{}.ps1".format(f)
+            if os.path.isfile(path_bat) and "bat" in accept:
                 bats.append(path_bat)
-            elif os.path.isfile(path_sh):
+            if os.path.isfile(path_ps1) and "ps1" in accept:
+                ps1s.append(path_ps1)
+            if os.path.isfile(path_sh) and "sh" in accept:
                 path_sh = subsystem_path(subsystem, path_sh)
                 shs.append(path_sh)
 
-    if bats and shs:
-        raise ConanException("Cannot wrap command with different envs, {} - {}".format(bats, shs))
+    if bool(bats) + bool(shs) + bool(ps1s) > 1:
+        raise ConanException("Cannot wrap command with different envs,"
+                             " {} - {} - {}".format(bats, shs, ps1s))
 
     if bats:
         launchers = " && ".join('"{}"'.format(b) for b in bats)
@@ -48,12 +56,16 @@ def environment_wrap_command(env_filenames, cmd, subsystem=None, cwd=None):
     elif shs:
         launchers = " && ".join('. "{}"'.format(f) for f in shs)
         return '{} && {}'.format(launchers, cmd)
+    elif ps1s:
+        # TODO: at the moment it only works with path without spaces
+        launchers = " ; ".join('"&\'{}\'"'.format(f) for f in ps1s)
+        return 'powershell.exe {} ; cmd /c {}'.format(launchers, cmd)
     else:
         return cmd
 
 
 class _EnvValue:
-    def __init__(self, name, value=_EnvVarPlaceHolder, separator=" ", path=False):
+    def __init__(self, name, value=None, separator=" ", path=False):
         self._name = name
         self._values = [] if value is None else value if isinstance(value, list) else [value]
         self._path = path
@@ -131,7 +143,6 @@ class _EnvValue:
                     values.append(placeholder.format(name=self._name))
             else:
                 if self._path:
-                    from conan.tools.microsoft.subsystems import subsystem_path
                     v = subsystem_path(subsystem, v)
                 values.append(v)
         if self._path:
@@ -143,6 +154,12 @@ class _EnvValue:
         previous_value = os.getenv(self._name)
         return self.get_str(previous_value, subsystem, pathsep)
 
+    def set_relative_base_folder(self, folder):
+        if not self._path:
+            return
+        self._values = [os.path.join(folder, v) if v != _EnvVarPlaceHolder else v
+                        for v in self._values]
+
 
 class Environment:
     def __init__(self):
@@ -181,16 +198,16 @@ class Environment:
         self._values[name] = _EnvValue(name, None)
 
     def append(self, name, value, separator=None):
-        self._values.setdefault(name, _EnvValue(name)).append(value, separator)
+        self._values.setdefault(name, _EnvValue(name, _EnvVarPlaceHolder)).append(value, separator)
 
     def append_path(self, name, value):
-        self._values.setdefault(name, _EnvValue(name, path=True)).append(value)
+        self._values.setdefault(name, _EnvValue(name, _EnvVarPlaceHolder, path=True)).append(value)
 
     def prepend(self, name, value, separator=None):
-        self._values.setdefault(name, _EnvValue(name)).prepend(value, separator)
+        self._values.setdefault(name, _EnvValue(name, _EnvVarPlaceHolder)).prepend(value, separator)
 
     def prepend_path(self, name, value):
-        self._values.setdefault(name, _EnvValue(name, path=True)).prepend(value)
+        self._values.setdefault(name, _EnvValue(name, _EnvVarPlaceHolder, path=True)).prepend(value)
 
     def remove(self, name, value):
         self._values[name].remove(value)
@@ -220,12 +237,16 @@ class Environment:
         return not self.__eq__(other)
 
     def vars(self, conanfile, scope="build"):
-        return EnvVars(conanfile, self, scope)
+        return EnvVars(conanfile, self._values, scope)
+
+    def set_relative_base_folder(self, folder):
+        for v in self._values.values():
+            v.set_relative_base_folder(folder)
 
 
 class EnvVars:
-    def __init__(self, conanfile, env, scope):
-        self._values = env._values  # {var_name: _EnvValue}, just a reference to the Environment
+    def __init__(self, conanfile, values, scope):
+        self._values = values  # {var_name: _EnvValue}, just a reference to the Environment
         self._conanfile = conanfile
         self._scope = scope
         self._subsystem = deduce_subsystem(conanfile, scope)
@@ -240,16 +261,34 @@ class EnvVars:
     def keys(self):
         return self._values.keys()
 
-    def get(self, name, default=None):
+    def get(self, name, default=None, variable_reference=None):
+        """
+        :param name: The name of the environment variable
+        :param default: The returned value if the variable doesn't exist, by default None
+        :param variable_reference: if specified, use a variable reference instead of the
+        pre-existing value of environment variable, where {name} can be used to refer to the
+        name of the variable.
+        """
         v = self._values.get(name)
         if v is None:
             return default
-        return v.get_value(self._subsystem, self._pathsep)
+        if variable_reference:
+            return v.get_str(variable_reference, self._subsystem, self._pathsep)
+        else:
+            return v.get_value(self._subsystem, self._pathsep)
 
-    def items(self):
-        """returns {str: str} (varname: value)"""
-        return {k: v.get_value(self._subsystem, self._pathsep)
-                for k, v in self._values.items()}.items()
+    def items(self, variable_reference=None):
+        """returns {str: str} (varname: value)
+         :param variable_reference: if specified, use a variable reference instead of the
+        pre-existing value of environment variable, where {name} can be used to refer to the
+        name of the variable.
+        """
+        if variable_reference:
+            return {k: v.get_str(variable_reference, self._subsystem, self._pathsep)
+                    for k, v in self._values.items()}.items()
+        else:
+            return {k: v.get_value(self._subsystem, self._pathsep)
+                    for k, v in self._values.items()}.items()
 
     @contextmanager
     def apply(self):
@@ -262,44 +301,48 @@ class EnvVars:
             os.environ.clear()
             os.environ.update(old_env)
 
-    def save_bat(self, filename, generate_deactivate=True):
+    def save_bat(self, file_location, generate_deactivate=True):
+        _, filename = os.path.split(file_location)
+        deactivate_file = "deactivate_{}".format(filename)
         deactivate = textwrap.dedent("""\
-            echo Capturing current environment in deactivate_{filename}
             setlocal
-            echo @echo off > "deactivate_{filename}"
-            echo echo Restoring environment >> "deactivate_{filename}"
+            echo @echo off > "%~dp0/{deactivate_file}"
+            echo echo Restoring environment >> "%~dp0/{deactivate_file}"
             for %%v in ({vars}) do (
                 set foundenvvar=
                 for /f "delims== tokens=1,2" %%a in ('set') do (
                     if /I "%%a" == "%%v" (
-                        echo set %%a=%%b>> "deactivate_{filename}"
+                        echo set "%%a=%%b">> "%~dp0/{deactivate_file}"
                         set foundenvvar=1
                     )
                 )
                 if not defined foundenvvar (
-                    echo set %%v=>> "deactivate_{filename}"
+                    echo set %%v=>> "%~dp0/{deactivate_file}"
                 )
             )
             endlocal
-            """).format(filename=os.path.basename(filename), vars=" ".join(self._values.keys()))
+            """).format(deactivate_file=deactivate_file, vars=" ".join(self._values.keys()))
         capture = textwrap.dedent("""\
             @echo off
+            chcp 65001 > nul
             {deactivate}
-            echo Configuring environment variables
             """).format(deactivate=deactivate if generate_deactivate else "")
         result = [capture]
         for varname, varvalues in self._values.items():
             value = varvalues.get_str("%{name}%", subsystem=self._subsystem, pathsep=self._pathsep)
-            result.append('set {}={}'.format(varname, value))
+            result.append('set "{}={}"'.format(varname, value))
 
         content = "\n".join(result)
-        save(filename, content)
+        # It is very important to save it correctly with utf-8, the Conan util save() is broken
+        os.makedirs(os.path.dirname(os.path.abspath(file_location)), exist_ok=True)
+        open(file_location, "w", encoding="utf-8").write(content)
 
-    def save_ps1(self, filename, generate_deactivate=True,):
+    def save_ps1(self, file_location, generate_deactivate=True,):
+        _, filename = os.path.split(file_location)
+        deactivate_file = "deactivate_{}".format(filename)
         deactivate = textwrap.dedent("""\
-            echo "Capturing current environment in deactivate_{filename}"
-
-            "echo `"Restoring environment`"" | Out-File -FilePath "deactivate_{filename}"
+            Push-Location $PSScriptRoot
+            "echo `"Restoring environment`"" | Out-File -FilePath "{deactivate_file}"
             $vars = (Get-ChildItem env:*).name
             $updated_vars = @({vars})
 
@@ -308,74 +351,88 @@ class EnvVars:
                 if ($var -in $vars)
                 {{
                     $var_value = (Get-ChildItem env:$var).value
-                    Add-Content "deactivate_{filename}" "`n`$env:$var = `"$var_value`""
+                    Add-Content "{deactivate_file}" "`n`$env:$var = `"$var_value`""
                 }}
                 else
                 {{
-                    Add-Content "deactivate_{filename}" "`nif (Test-Path env:$var) {{ Remove-Item env:$var }}"
+                    Add-Content "{deactivate_file}" "`nif (Test-Path env:$var) {{ Remove-Item env:$var }}"
                 }}
             }}
+            Pop-Location
         """).format(
-            filename=os.path.basename(filename),
+            deactivate_file=deactivate_file,
             vars=",".join(['"{}"'.format(var) for var in self._values.keys()])
         )
 
         capture = textwrap.dedent("""\
             {deactivate}
-            echo "Configuring environment variables"
         """).format(deactivate=deactivate if generate_deactivate else "")
         result = [capture]
         for varname, varvalues in self._values.items():
             value = varvalues.get_str("$env:{name}", subsystem=self._subsystem, pathsep=self._pathsep)
             if value:
+                value = value.replace('"', '`"')  # escape quotes
                 result.append('$env:{}="{}"'.format(varname, value))
             else:
                 result.append('if (Test-Path env:{0}) {{ Remove-Item env:{0} }}'.format(varname))
 
         content = "\n".join(result)
-        save(filename, content)
+        # It is very important to save it correctly with utf-16, the Conan util save() is broken
+        # and powershell uses utf-16 files!!!
+        os.makedirs(os.path.dirname(os.path.abspath(file_location)), exist_ok=True)
+        open(file_location, "w", encoding="utf-16").write(content)
 
-    def save_sh(self, filename, generate_deactivate=True):
+    def save_sh(self, file_location, generate_deactivate=True):
+        filepath, filename = os.path.split(file_location)
+        deactivate_file = os.path.join(filepath, "deactivate_{}".format(filename))
         deactivate = textwrap.dedent("""\
-           echo Capturing current environment in deactivate_{filename}
-           echo echo Restoring environment >> deactivate_{filename}
+           echo "echo Restoring environment" > "{deactivate_file}"
            for v in {vars}
            do
-               value=$(printenv $v)
-               if [ -n "$value" ]
+               is_defined="true"
+               value=$(printenv $v) || is_defined="" || true
+               if [ -n "$value" ] || [ -n "$is_defined" ]
                then
-                   echo export "$v=$value" >> deactivate_{filename}
+                   echo export "$v='$value'" >> "{deactivate_file}"
                else
-                   echo unset $v >> deactivate_{filename}
+                   echo unset $v >> "{deactivate_file}"
                fi
            done
-           """.format(filename=os.path.basename(filename), vars=" ".join(self._values.keys())))
+           """.format(deactivate_file=deactivate_file, vars=" ".join(self._values.keys())))
         capture = textwrap.dedent("""\
               {deactivate}
-              echo Configuring environment variables
               """).format(deactivate=deactivate if generate_deactivate else "")
         result = [capture]
         for varname, varvalues in self._values.items():
             value = varvalues.get_str("${name}", self._subsystem, pathsep=self._pathsep)
+            value = value.replace('"', '\\"')
             if value:
                 result.append('export {}="{}"'.format(varname, value))
             else:
                 result.append('unset {}'.format(varname))
 
         content = "\n".join(result)
-        save(filename, content)
+        save(file_location, content)
 
     def save_script(self, filename):
         name, ext = os.path.splitext(filename)
         if ext:
             is_bat = ext == ".bat"
+            is_ps1 = ext == ".ps1"
         else:  # Need to deduce it automatically
             is_bat = self._subsystem == WINDOWS
-            filename = filename + (".bat" if is_bat else ".sh")
+            is_ps1 = self._conanfile.conf.get("tools.env.virtualenv:powershell", check_type=bool)
+            if is_ps1:
+                filename = filename + ".ps1"
+                is_bat = False
+            else:
+                filename = filename + (".bat" if is_bat else ".sh")
 
         path = os.path.join(self._conanfile.generators_folder, filename)
         if is_bat:
             self.save_bat(path)
+        elif is_ps1:
+            self.save_ps1(path)
         else:
             self.save_sh(path)
 
@@ -450,12 +507,17 @@ class ProfileEnvironment:
                 else:
                     pattern, name = None, pattern_name[0]
 
+                # strip whitespaces before/after =
+                # values are not strip() unless they are a path, to preserve potential whitespaces
+                name = name.strip()
+
                 # When loading from profile file, latest line has priority
                 env = Environment()
                 if method == "unset":
                     env.unset(name)
                 else:
-                    if value.startswith("(path)"):
+                    if value.strip().startswith("(path)"):
+                        value = value.strip()
                         value = value[6:]
                         method = method + "_path"
                     getattr(env, method)(name, value)
