@@ -11,7 +11,8 @@ from conans.util.log import logger
 
 def cmd_build(app, conanfile_path, base_path, source_folder, build_folder, package_folder,
               install_folder, test=False, should_configure=True, should_build=True,
-              should_install=True, should_test=True):
+              should_install=True, should_test=True, layout_source_folder=None,
+              layout_build_folder=None, conf=None):
     """ Call to build() method saved on the conanfile.py
     param conanfile_path: path to a conanfile.py
     """
@@ -28,6 +29,8 @@ def cmd_build(app, conanfile_path, base_path, source_folder, build_folder, packa
                              "requirements and generators from '%s' file"
                              % (CONANFILE, CONANFILE, CONANFILE_TXT))
 
+    if conf:
+        conan_file.conf.compose_conf(conf)
     if test:
         try:
             conan_file.requires.add_ref(test)
@@ -44,11 +47,7 @@ def cmd_build(app, conanfile_path, base_path, source_folder, build_folder, packa
         #  Only base_path and conanfile_path will remain
         if hasattr(conan_file, "layout"):
             conanfile_folder = os.path.dirname(conanfile_path)
-            conan_file.folders.set_base_build(conanfile_folder)
-            conan_file.folders.set_base_source(conanfile_folder)
-            conan_file.folders.set_base_package(conanfile_folder)
-            conan_file.folders.set_base_generators(conanfile_folder)
-            conan_file.folders.set_base_install(conanfile_folder)
+            conan_file.folders.set_base_folders(conanfile_folder, layout_build_folder)
         else:
             conan_file.folders.set_base_build(build_folder)
             conan_file.folders.set_base_source(source_folder)
