@@ -226,6 +226,17 @@ class OptionsValues(object):
     def __contains__(self, item):
         return item in self._package_values
 
+    def get_safe(self, attr):
+        if attr not in self._package_values:
+            return None
+        return getattr(self._package_values, attr)
+
+    def rm_safe(self, attr):
+        try:
+            delattr(self._package_values, attr)
+        except ConanException:
+            pass
+
     def __getitem__(self, item):
         return self._reqs_options.setdefault(item, PackageOptionValues())
 
@@ -420,6 +431,12 @@ class PackageOptions(object):
     def get_safe(self, field, default=None):
         return self._data.get(field, default)
 
+    def rm_safe(self, field):
+        try:
+            delattr(self, field)
+        except ConanException:
+            pass
+
     def validate(self):
         for child in self._data.values():
             child.validate()
@@ -579,6 +596,9 @@ class Options(object):
         except ConanException:
             pass
 
+    def rm_safe(self, field):
+        self._package_options.rm_safe(field)
+
     @property
     def values(self):
         result = OptionsValues()
