@@ -1,5 +1,6 @@
-from conans.errors import ConanException
 import yaml
+
+from conans.errors import ConanException
 from conans.model.values import Values
 
 
@@ -25,11 +26,17 @@ def undefined_value(name):
 
 
 class SettingsItem(object):
+    """ represents a setting value and its child info, which could be:
+    - A range of valid values: [Debug, Release] (for settings.compiler.runtime of VS)
+    - "ANY", as string to accept any value
+    - List ["None", "ANY"] to accept None or any value
+    - A dict {subsetting: definition}, e.g. {version: [], runtime: []} for VS
+    """
     def __init__(self, definition, name):
-        self._name = name
-        self._value = None
-        self._definition = {}
+        self._name = name  # settings.compiler
+        self._value = None  # gcc
         if isinstance(definition, dict):
+            self._definition = {}
             # recursive
             for k, v in definition.items():
                 k = str(k)
@@ -38,7 +45,7 @@ class SettingsItem(object):
             self._definition = "ANY"
         else:
             # list or tuple of possible values
-            self._definition = sorted(str(v) for v in definition)
+            self._definition = [str(v) for v in definition]
 
     def __contains__(self, value):
         return value in (self._value or "")
@@ -79,13 +86,16 @@ class SettingsItem(object):
         return self.__bool__()
 
     def __str__(self):
-        return self._value
+        return str(self._value)
+
+    def _not_any(self):
+        return self._definition != "ANY" and "ANY" not in self._definition
 
     def __eq__(self, other):
         if other is None:
             return self._value is None
         other = str(other)
-        if self._definition != "ANY" and other not in self.values_range:
+        if self._not_any() and other not in self.values_range:
             raise ConanException(bad_value_msg(self._name, other, self.values_range))
         return other == self.__str__()
 
@@ -98,7 +108,7 @@ class SettingsItem(object):
         """
         try:
             self._get_child(self._value).remove(item)
-        except:
+        except Exception:
             pass
 
     def remove(self, values):
@@ -108,10 +118,13 @@ class SettingsItem(object):
             v = str(v)
             if isinstance(self._definition, dict):
                 self._definition.pop(v, None)
-            elif self._definition != "ANY":
-                if v in self._definition:
-                    self._definition.remove(v)
-        if self._value is not None and self._value not in self._definition:
+            elif self._definition == "ANY":
+                if v == "ANY":
+                    self._definition = []
+            elif v in self._definition:
+                self._definition.remove(v)
+
+        if self._value is not None and self._value not in self._definition and self._not_any():
             raise ConanException(bad_value_msg(self._name, self._value, self.values_range))
 
     def _get_child(self, item):
@@ -138,7 +151,7 @@ class SettingsItem(object):
         value = str(value)
         try:
             return self._definition[value]
-        except:
+        except Exception:
             raise ConanException(bad_value_msg(self._name, value, self.values_range))
 
     @property
@@ -148,7 +161,7 @@ class SettingsItem(object):
     @value.setter
     def value(self, v):
         v = str(v)
-        if self._definition != "ANY" and v not in self._definition:
+        if self._not_any() and v not in self.values_range:
             raise ConanException(bad_value_msg(self._name, v, self.values_range))
         self._value = v
 
@@ -156,7 +169,7 @@ class SettingsItem(object):
     def values_range(self):
         try:
             return sorted(list(self._definition.keys()))
-        except:
+        except Exception:
             return self._definition
 
     @property
@@ -174,29 +187,44 @@ class SettingsItem(object):
     def validate(self):
         if self._value is None and "None" not in self._definition:
             raise undefined_value(self._name)
-
         if isinstance(self._definition, dict):
-            self._definition[self._value].validate()
+            key = "None" if self._value is None else self._value
+            self._definition[key].validate()
 
 
 class Settings(object):
     def __init__(self, definition=None, name="settings", parent_value=None):
+        if parent_value == "None" and definition:
+            raise ConanException("settings.yml: None setting can't have subsettings")
         definition = definition or {}
         self._name = name  # settings, settings.compiler
         self._parent_value = parent_value  # gcc, x86
         self._data = {str(k): SettingsItem(v, "%s.%s" % (name, k))
                       for k, v in definition.items()}
 
-    def get_safe(self, name):
+    def get_safe(self, name, default=None):
         try:
             tmp = self
             for prop in name.split("."):
                 tmp = getattr(tmp, prop, None)
         except ConanException:
-            return None
+            return default
         if tmp is not None and tmp.value and tmp.value != "None":  # In case of subsettings is None
             return str(tmp)
-        return None
+        return default
+
+    def rm_safe(self, name):
+        try:
+            tmp = self
+            attr_ = name
+            if "." in name:
+                fields = name.split(".")
+                attr_ = fields.pop()
+                for prop in fields:
+                    tmp = getattr(tmp, prop)
+            delattr(tmp, attr_)
+        except ConanException:
+            pass
 
     def copy(self):
         """ deepcopy, recursive
@@ -218,7 +246,10 @@ class Settings(object):
 
     @staticmethod
     def loads(text):
-        return Settings(yaml.load(text) or {})
+        try:
+            return Settings(yaml.safe_load(text) or {})
+        except (yaml.YAMLError, AttributeError) as ye:
+            raise ConanException("Invalid settings.yml format: {}".format(ye))
 
     def validate(self):
         for field in self.fields:
@@ -278,9 +309,9 @@ class Settings(object):
     def iteritems(self):
         return self.values_list
 
-    @values_list.setter
-    def values_list(self, vals):
+    def update_values(self, vals):
         """ receives a list of tuples (compiler.version, value)
+        This is more an updated than a setter
         """
         assert isinstance(vals, list), vals
         for (name, value) in vals:
@@ -293,7 +324,7 @@ class Settings(object):
     @values.setter
     def values(self, vals):
         assert isinstance(vals, Values)
-        self.values_list = vals.as_list()
+        self.update_values(vals.as_list())
 
     def constraint(self, constraint_def):
         """ allows to restrict a given Settings object with the input of another Settings object
@@ -315,6 +346,8 @@ class Settings(object):
             other_field_def = constraint_def[field]
             if other_field_def is None:  # Means leave it as is
                 continue
+            if isinstance(other_field_def, str):
+                other_field_def = [other_field_def]
 
             values_to_remove = []
             for value in config_item.values_range:  # value = "Visual Studio"
